# encoding: UTF-8
# 
# Usage:
# save your data to ./input/<input_file> as csv (translation,rotation,count)
# ruby tomo.rb <input_file> <output_resolution> [<verfahren 'first' or 'second'>] [<comment>] [<angle correction>]
#
#

require 'matrix'
require 'rubygems'
require 'ruby-progressbar'

require './lib/data_object.rb'
require './lib/load_data.rb'
require './lib/load_data_alt.rb'
require './lib/filter.rb'
require './lib/matrixausgabe.rb'
require './lib/chunky_png.rb'
require './lib/adjust_measurement.rb'

if ARGV[4]
  $angle_correction = ARGV[4].to_f
else
  $angle_correction = 1.25
end

puts "Winkelkorrektur: " + $angle_correction.to_s

data = load_data ARGV[0]

 data = adjust_measurement data

output_res = ARGV[1].to_i
png = ChunkyPNG::Image.new(output_res, output_res, ChunkyPNG::Color::TRANSPARENT)
log = ""

if ARGV[3]
log = log + "=== Kommentar ===\n"
log = log + ARGV[3] + "\n"
end
log = log + "=== Input-Daten ===\n"
log = log + ARGV[0] + "\n"
log = log + "=== Messpunkte ===\n"
log = log + data.length.to_s + "\n"
log = log + "=== Winkelkorrektur ===\n"
log = log + $angle_correction.to_s + "\n"
log = log + "=== Filter ===\n"
log = log + File.open("./lib/filter.rb", "rb").read + "\n"


# Leeres Abbildungs-Array erzeugen mit der eingestellten Ausfloesung

projection = Array.new(output_res) { Array.new(output_res) }
for x in 0..output_res-1
  for y in 0..output_res-1
    projection[x][y] = 0
  end
end

progressbar = ProgressBar.create(:format => '%E|%b%i|%p%%', :progress_mark  => '･', :remainder_mark => ' ')

if ARGV[2] == "second"
  #Abbilden
  #(10..180).step(4) do |winkel|
  #for l in 0..data.length-1
  #puts l
  for i in 0..output_res-1
    progressbar.progress = (100*i/(output_res-1.0))
    x=(i-(output_res/2.0)).to_f
    for j in 0..output_res-1
      y= -(j-(output_res/2.0)).to_f
      if x**2+y**2 <= (output_res/2.0)**2
        r = Vector[x, y]
        for k in 0..data.length-1
        # if data[k].rot < winkel
          u = Vector[Math.cos(data[k].rot), Math.sin(data[k].rot)] # Richtungvektor des Messpunktes
          d = (u.covector*r)[0] # projection des Ortsvektors der Abbildung auf den Messvektor
          projection[i][j] += filter(d/(output_res*1.0)-data[k].trans/2.0, data[k].count, Math.sqrt(data.length).to_i )
        #end
        end
      end
    end
  end
  #draw_array( png, projection, 0, 0, output_res, output_res)
  #file_name = Time.now.strftime("%Y-%m-%d-%H-%M-%S-%L")+l.to_s
  #png.save('./output/succ/' + file_name + '.png', :interlace => true)
  #end

elsif ARGV[2] == "first"
  
  a=output_res
   #Eingangsaufläsung
  b = Math.sqrt(data.length).to_i
  #filterkonstante=3.05
  #belichtung = 30
  #filterkonstante=2.9
  #belichtung = 30
  filterkonstante=3.5

  $vector = Array.new
  load_data_alt data, b
  
  winkel=$vector.length

  #Eingang "Belichten"
  for i in 0..$vector.length-1
    for j in 0..$vector[i].length-1
      #$vector[i][j] = Math.exp($vector[i][j]/40)
      $vector[i][j] = $vector[i][j]
    end
  end

  #Eingang filtern
  filter = Array.new($vector[0].length)
  #puts $vector[0].length

  for i in 0..$vector.length-1
    for j in 0..filter.length-1
      filter[j] = 0
    end
    for j in 0..$vector[i].length-1
      for k in 0..$vector[i].length-1
        unless k==j
          filter[k] -= $vector[i][j]/(filterkonstante*(k-j)**2)
  #        filter[k] -= $vector[i][j]* Math.exp(-(k-j)**2) * (1-((k-j)*1.2)**2)   
        end
      end
    end
    for j in 0..filter.length-1
      #puts $vector[i][j], filter[j]
      $vector[i][j] += filter[j]
    end
  end

  projection_count = Array.new
  
  for i in 0..a-1
    projection_count[i] = Array.new
    for j in 0..a-1
      projection[i][j] = 0.0
      projection_count[i][j] = 0
    end
  end

  #Abbilden

  for i in 0..a-1
    #puts i.to_s
    progressbar.progress = (99*(i)/output_res)+1
    for j in 0..a-1

      if (i-a/2.0)**2 +(j-a/2.0)**2<(a/2.0-1)**2
        for k in 0..$vector.length-1
        #puts i.to_s + j.to_s + k.to_s
          phi = Math::PI*k*$angle_correction/winkel
          dx = ((i-(a/2))*Math.cos(phi) + (j-(a/2))*Math.sin(phi) )*b/a #-4.5
          if dx > -(b/2)+1 && dx < (b/2)-1
            projection[i][a-j-1] += ($vector[k][(b/2)+dx.ceil] - $vector[k][(b/2)+dx.floor]) * (dx - dx.floor) + $vector[k][(b/2)+dx.floor]
            projection_count[i][a-j-1] +=1
          end
        end
      end
    end
  end
  
  #Enlichten

  for i in 0..a-1
    for j in 0..a-1
      if projection_count[i][j] > 0
        projection[i][j] = projection[i][j]/projection_count[i][j]
      end
      if projection[i][j] < 0
        projection[i][j] = 0
      end
      #if projection[i][j] > 255
      #  puts projection[i][j]
      #end
    end
  end


end

#speichern

draw_array( png, projection, 0, 0, output_res, output_res)
file_name = Time.now.strftime("%Y-%m-%d-%H-%M-%S-%L")

png.save('./output/output' + file_name + '.png', :interlace => true)

File.open('./output/output' + file_name + '.log', 'w+') { |file| file.write(log) }

progressbar.finish
puts "done"
